/** @file sp/spFile.h
 */

#ifndef __SPFILE_H
#define __SPFILE_H

#include <sp/spDefs.h>

#ifdef __cplusplus
extern "C" {
#endif

#if !defined(BYTE_ORDER)
#if defined(__i386__) || defined(__alpha__) || defined(__alpha) \
    || defined(_M_IX86) || defined(_M_IA64) || defined(_M_X64) || defined(_M_ALPHA) \
    || defined(_AMD64_)	|| defined(_M_AMD64) \
    || ((defined(_M_ARM) || defined(_M_ARM64)) && defined(_WIN32)) \
    || (defined(__mips__) && (defined(MIPSEL) || defined (__MIPSEL__))) \
    || (defined(__BYTE_ORDER__) && defined(__ORDER_LITTLE_ENDIAN__) && (__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__))
#define BYTE_ORDER 1234
#else
#define BYTE_ORDER 4321
#endif
#endif

#ifndef LITTLE_ENDIAN
#define LITTLE_ENDIAN 1234
#endif
#ifndef BIG_ENDIAN
#define BIG_ENDIAN 4321
#endif
    
/** @~english @defgroup spFileGroup File I/O */

/** @addtogroup spFileGroup
 *  @{ */  /*----@addtogroup spFileGroup----*/

#if defined(MACOS)
#pragma import on
#endif

/** @~english @name File Size
 *  @{ */

/** @~english Gets the size of the binary file. */
extern spFileOffset spGetFileSize(const char *filename, long headsize);
/** @~english Gets the size of the text file.
 *  This size corresponds to the number of valid lines excluding comment lines which begin with '#' in the file. */
extern long spGetTextFileSize(const char *filename);
/** @~english Skips the header of the binary file. The origin of this function call is always beginning of a file. */
extern void spFSkipHeader(long headsize, FILE *fp);
/** @~english Skips a part of the binary file. The origin of this function call is the current position of the file pointer. */
extern long spFSkipByte(long len, FILE *fp);

/** @~english Gets the length of the signal file. */
#define spGetSignalLength(filename, headsize, type) (spGetFileSize(filename, headsize) / (spFileOffset)sizeof(type))
/** @~english Skips a part of the signal file. The origin of this function call is the current position of the file pointer. */
#define spFSkipData(len, fp, type) (spFSkipByte((len) * (long)sizeof(type), fp) / (long)sizeof(type))

/** @~english @} */

/** @~english @name Data Shift
 *  @{ */
 
/** @~english Shifts contents of the buffer of short type. */
extern void spShiftShort(short *data, long length, long shift);
/** @~english Shifts contents of the buffer of long type. */
extern void spShiftLong(long *data, long length, long shift);
/** @~english Shifts contents of the buffer of float type. */
extern void spShiftFloat(float *data, long length, long shift);
/** @~english Shifts contents of the buffer of double type. */
extern void spShiftDouble(double *data, long length, long shift);
    
/** @~english @} */

/** @~english @name Data Swap
 *  @{ */
 
/** @~english Byte-swaps all elements of the buffer for any type. */
extern void spSwapByte(void *data, long length, int samp_byte);
/** @~english Byte-swaps all elements of the buffer of short type. */
extern void spSwapShort(short *data, long length);
/** @~english Byte-swaps all elements of the buffer of long type. */
extern void spSwapLong(long *data, long length);
/** @~english Byte-swaps all elements of the buffer of spLong32 type. */
extern void spSwapLong32(spLong32 *data, long length);
/** @~english Byte-swaps all elements of the buffer of spULong32 type. */
extern void spSwapULong32(spULong32 *data, long length);
/** @~english Byte-swaps all elements of the buffer of float type. */
extern void spSwapFloat(float *data, long length);
/** @~english Byte-swaps all elements of the buffer of double type. */
extern void spSwapDouble(double *data, long length);

#ifdef SP_SUPPORT_64BIT_INT
/** @~english Byte-swaps all elements of the buffer of spLong64 (64-bit long) type. */
extern void spSwapLong64(spLong64 *data, long length);
/** @~english Byte-swaps all elements of the buffer of spULong64 (64-bit unsigned long) type. */
extern void spSwapULong64(spULong64 *data, long length);
#endif
    
/** @~english @} */

/** @internal Byte-swaps all elements of the buffer of 64bit long type represented by 4*char */
extern void spSwapLongC64(char *data_c64, long length);

/** @~english @name Read
 *  @{ */
 
/** @~english Reads data of short type from the binary file. */
extern long spFReadShort(short *data, long length, int swap, FILE *fp);
/** @~english Reads data of long type from the binary file. */
extern long spFReadLong(long *data, long length, int swap, FILE *fp);
/** @~english Reads data of 32-bit integer from the binary file. */
extern long spFReadLong32(long *data, long length, int swap, FILE *fp);
/** @~english Reads data of 32-bit unsigned integer from the binary file. */
extern long spFReadULong32(unsigned long *data, long length, int swap, FILE *fp);
/** @~english Reads data of 32-bit integer from the binary file to the 32-bit integer buffer. */
extern long spFReadLong32Direct(spLong32 *data, long length, int swap, FILE *fp);
/** @~english Reads data of 32-bit unsigned integer from the binary file to the 32-bit unsigned integer buffer. */
extern long spFReadULong32Direct(spULong32 *data, long length, int swap, FILE *fp);
/** @~english Reads data of 24-bit integer from the binary file. */
extern long spFReadLong24(long *data, long length, int swap, FILE *fp);
/** @~english Reads data of 24-bit unsigned integer from the binary file. */
extern long spFReadULong24(unsigned long *data, long length, int swap, FILE *fp);
#ifdef SP_SUPPORT_64BIT_INT
/** @~english Reads data of 64-bit integer from the binary file. */
extern long spFReadLong64(spLong64 *data, long length, int swap, FILE *fp);
/** @~english Reads data of 64-bit unsigned integer from the binary file. */
extern long spFReadULong64(spULong64 *data, long length, int swap, FILE *fp);
#endif
/** @~english Reads data of float type from the binary file. */
extern long spFReadFloatWeighted(float *data, long length, float weight, int swap, FILE *fp);
/** @~english Reads data of float type from the binary file. */
extern long spFReadFloat(float *data, long length, int swap, FILE *fp);
/** @~english Reads data of double type from the binary file. */
extern long spFReadDoubleWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Reads data of double type from the binary file. */
extern long spFReadDouble(double *data, long length, int swap, FILE *fp);
/** @~english Reads data of 8-bit unsigned integer from the binary file. */
extern long spFReadByte(short *data, long length, FILE *fp);
/** @~english Reads data of 8-bit signed integer from the binary file. */
extern long spFReadSignedByte(short *data, long length, FILE *fp);
/** @~english Reads data of u-law from the binary file. */
extern long spFReadULaw(short *data, long length, FILE *fp);
/** @~english Reads data of a-law from the binary file. */
extern long spFReadALaw(short *data, long length, FILE *fp);

/** @~english Reads data of INT8 type on Windows from the binary file. */
extern long spFReadINT8(spINT8 *data, long length, FILE *fp);
/** @~english Reads data of INT16 type on Windows from the binary file. */
extern long spFReadINT16(spINT16 *data, long length, FILE *fp);
/** @~english Reads data of BYTE type on Windows from the binary file. */
extern long spFReadBYTE(spBYTE *data, long length, FILE *fp);
/** @~english Reads data of WORD type on Windows from the binary file. */
extern long spFReadWORD(spWORD *data, long length, FILE *fp);
/** @~english Reads data of LONG type on Windows from the binary file. */
extern long spFReadLONG(spLONG *data, long length, FILE *fp);
/** @~english Reads data of ULONG type on Windows from the binary file. */
extern long spFReadULONG(spULONG *data, long length, FILE *fp);
/** @~english Reads data of DWORD type on Windows from the binary file. */
#define spFReadDWORD(data, length, fp) spFReadULONG(data, length, fp)

/** @~english Reads data of short type to double array from the binary file. */
extern long spFReadShortToDoubleWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Reads data of short type to double array from the binary file. */
extern long spFReadShortToDouble(double *data, long length, int swap, FILE *fp);
/** @~english Reads data of long type to double array from the binary file. */
extern long spFReadLongToDoubleWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Reads data of long type to double array from the binary file. */
extern long spFReadLongToDouble(double *data, long length, int swap, FILE *fp);
/** @~english Reads data of 32-bit integer to double array from the binary file. */
extern long spFReadLong32ToDoubleWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Reads data of 32-bit integer to double array from the binary file. */
extern long spFReadLong32ToDouble(double *data, long length, int swap, FILE *fp);
/** @~english Reads data of 24-bit integer to double array from the binary file. */
extern long spFReadLong24ToDoubleWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Reads data of 24-bit integer to double array from the binary file. */
extern long spFReadLong24ToDouble(double *data, long length, int swap, FILE *fp);
/** @~english Reads data of float type to double array from the binary file. */
extern long spFReadFloatToDoubleWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Reads data of float type to double array from the binary file. */
extern long spFReadFloatToDouble(double *data, long length, int swap, FILE *fp);
/** @~english Reads data of specified bit format to double array from the binary file. */
extern long spFReadBitToDoubleWeighted(double *data, long length, double weight, int samp_bit, int swap, FILE *fp);
/** @~english Reads data of specified bit format to double array from the binary file. */
extern long spFReadBitToDouble(double *data, long length, double max, int samp_bit, int swap, FILE *fp);
    
/** @~english @} */

/* max: Maximum allowable value of input type is mapped to this value. */
/** @internal */
extern long spFReadFrameBit(double *data, long length, long shift, double max, int samp_bit, int swap, FILE *fp);

/** @~english @name Write
 *  @{ */
 
/** @~english Writes data of short type to the binary file. */
extern long spFWriteShort(short *data, long length, int swap, FILE *fp);
/** @~english Writes data of long type to the binary file. */
extern long spFWriteLong(long *data, long length, int swap, FILE *fp);
/** @~english Writes data of long type to the binary file as that of 32-bit integer. */
extern long spFWriteLong32(long *data, long length, int swap, FILE *fp);
/** @~english Writes data of long type to the binary file as that of 32-bit unsigned integer. */
extern long spFWriteULong32(unsigned long *data, long length, int swap, FILE *fp);
/** @~english Writes data of spLong32 type to the binary file. */
extern long spFWriteLong32Direct(spLong32 *data, long length, int swap, FILE *fp);
/** @~english Writes data of spULong32 type to the binary file. */
extern long spFWriteULong32Direct(spULong32 *data, long length, int swap, FILE *fp);
/** @~english Writes data of long type to the binary file as that of 24-bit integer. */
extern long spFWriteLong24(long *data, long length, int swap, FILE *fp);
/** @~english Writes data of unsigned long type to the binary file as that of 24-bit unsigned integer. */
extern long spFWriteULong24(unsigned long *data, long length, int swap, FILE *fp);
#ifdef SP_SUPPORT_64BIT_INT
/** @~english Writes data of spLong type to the binary file. */
extern long spFWriteLong64(spLong64 *data, long length, int swap, FILE *fp);
/** @~english Writes data of spULong type to the binary file. */
extern long spFWriteULong64(spULong64 *data, long length, int swap, FILE *fp);
#endif
/** @~english Writes data of float type to the binary file. */
extern long spFWriteFloatWeighted(float *data, long length, float weight, int swap, FILE *fp);
/** @~english Writes data of float type to the binary file. */
extern long spFWriteFloat(float *data, long length, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file. */
extern long spFWriteDoubleWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file. */
extern long spFWriteDouble(double *data, long length, int swap, FILE *fp);
/** @~english Writes data of short type to the binary file as that of 8-bit unsigned integer. */
extern long spFWriteByte(short *data, long length, FILE *fp);
/** @~english Writes data of short type to the binary file as that of 8-bit signed integer. */
extern long spFWriteSignedByte(short *data, long length, FILE *fp);
/** @~english Writes data of short type to the binary file as that of u-law. */
extern long spFWriteULaw(short *data, long length, FILE *fp);
/** @~english Writes data of short type to the binary file as that of a-law. */
extern long spFWriteALaw(short *data, long length, FILE *fp);

/** @~english Writes data as INT8 type on Windows to the binary file. */
extern long spFWriteINT8(spINT8 *data, long length, FILE *fp);
/** @~english Writes data as INT16 type on Windows to the binary file. */
extern long spFWriteINT16(spINT16 *data, long length, FILE *fp);
/** @~english Writes data as BYTE type on Windows to the binary file. */
extern long spFWriteBYTE(spBYTE *data, long length, FILE *fp);
/** @~english Writes data as WORD type on Windows to the binary file. */
extern long spFWriteWORD(spWORD *data, long length, FILE *fp);
/** @~english Writes data as LONG type on Windows to the binary file. */
extern long spFWriteLONG(spLONG *data, long length, FILE *fp);
/** @~english Writes data as ULONG type on Windows to the binary file. */
extern long spFWriteULONG(spULONG *data, long length, FILE *fp);
/** @~english Writes data as DWORD type on Windows to the binary file. */
#define spFWriteDWORD(data, length, fp) spFWriteULONG(data, length, fp)
    
/** @~english Writes data of double type to the binary file as that of short type. */
extern long spFWriteDoubleToShortWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of short type. */
extern long spFWriteDoubleToShort(double *data, long length, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of long type. */
extern long spFWriteDoubleToLongWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of long type. */
extern long spFWriteDoubleToLong(double *data, long length, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of 32-bit integer. */
extern long spFWriteDoubleToLong32Weighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of 32-bit integer. */
extern long spFWriteDoubleToLong32(double *data, long length, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of 24-bit integer. */
extern long spFWriteDoubleToLong24Weighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of 24-bit integer. */
extern long spFWriteDoubleToLong24(double *data, long length, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of float type. */
extern long spFWriteDoubleToFloatWeighted(double *data, long length, double weight, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of float type. */
extern long spFWriteDoubleToFloat(double *data, long length, int swap, FILE *fp);
/** @~english Writes data of double type to the binary file as that of specified bit format. */
extern long spFWriteDoubleToBitWeighted(double *data, long length, double weight, int samp_bit, int swap, FILE *fp);
/* max: This value is converted into the maximum allowable value of output type.
 * (e.g. the value is converted into 32768 in 16-bit case, 1.0 in floating cases. 1.0 means no conversion.) */
/** @~english Writes data of double type to the binary file as that of specified bit format. */
extern long spFWriteDoubleToBit(double *data, long length, double max, int samp_bit, int swap, FILE *fp);
/** @~english Writes data of long type to the binary file as that of short type. */
extern long spFWriteLong32ToShort(long *data, long length, int swap, FILE *fp);
/** @~english Writes data of long type to the binary file as that of short type. */
extern long spFWriteLong24ToShort(long *data, long length, int swap, FILE *fp);

/** @~english @} */

/** @~english @name Interleave and Deinterleave
 *  @{ */

/** @~english Interleaves data. Use spInterleaveData() instead of this function. */
extern long _spInterleaveData(char *ddata, long dlength, char *idata, long ilength, int num_channel, int samp_byte);
/** @~english Interleaves data.
 *  For example, for stereo data containing L and R, this function reorders data of LLL...RRR... into that of LRLRLR... .
 *  @param[in] ddata The input buffer.
 *  @param[in] dlength The number of elements of the input buffer.
 *  @param[in] idata The output buffer.
 *  @param[in] ilength The number of elements of the output buffer.
 *  @param[in] num_channel The number of channels.
 *  @param[in] type The type of data such as short, long, float or double.
 *  @return Returns the number of elements processed. */
#define spInterleaveData(ddata, dlength, idata, ilength, num_channel, type) \
_spInterleaveData((char *)(ddata), dlength, (char *)(idata), ilength, num_channel, sizeof(type))
/** @~english Interleaves data of short type. */
#define spInterleaveShort(ddata, dlength, idata, ilength, num_channel) \
spInterleaveData(ddata, dlength, idata, ilength, num_channel, short)
/** @~english Interleaves data of long type. */
#define spInterleaveLong(ddata, dlength, idata, ilength, num_channel) \
spInterleaveData(ddata, dlength, idata, ilength, num_channel, long)
/** @~english Interleaves data of spLong32 type. */
#define spInterleaveLong32(ddata, dlength, idata, ilength, num_channel) \
spInterleaveData(ddata, dlength, idata, ilength, num_channel, spLong32)
/** @~english Interleaves data of float type. */
#define spInterleaveFloat(ddata, dlength, idata, ilength, num_channel) \
spInterleaveData(ddata, dlength, idata, ilength, num_channel, float)
/** @~english Interleaves data of double type. */
#define spInterleaveDouble(ddata, dlength, idata, ilength, num_channel) \
spInterleaveData(ddata, dlength, idata, ilength, num_channel, double)

/** @~english Deinterleaves data. Use spDeinterleaveData() instead of this function. */
extern long _spDeinterleaveData(char *idata, long ilength, char *ddata, long dlength, int num_channel, int samp_byte);
/** @~english Deinterleaves data. 
 *  For example, for stereo data containing L and R, this function reorders data of LRLRLR... into that of LLL...RRR... .
 *  @param[in] idata The input buffer.
 *  @param[in] ilength The number of elements of the input buffer.
 *  @param[in] ddata The output buffer.
 *  @param[in] dlength The number of elements of the output buffer.
 *  @param[in] num_channel The number of channels.
 *  @param[in] type The type of data such as short, long, float or double.
 *  @return Returns the number of elements processed. */
#define spDeinterleaveData(idata, ilength, ddata, dlength, num_channel, type) \
_spDeinterleaveData((char *)(idata), ilength, (char *)(ddata), dlength, num_channel, sizeof(type))
/** @~english Deinterleaves data of short type. */
#define spDeinterleaveShort(idata, ilength, ddata, dlength, num_channel) \
spDeinterleaveData(idata, ilength, ddata, dlength, num_channel, short)
/** @~english Deinterleaves data of long type. */
#define spDeinterleaveLong(idata, ilength, ddata, dlength, num_channel) \
spDeinterleaveData(idata, ilength, ddata, dlength, num_channel, long)
/** @~english Deinterleaves data of spLong type. */
#define spDeinterleaveLong32(idata, ilength, ddata, dlength, num_channel) \
spDeinterleaveData(idata, ilength, ddata, dlength, num_channel, spLong32)
/** @~english Deinterleaves data of float type. */
#define spDeinterleaveFloat(idata, ilength, ddata, dlength, num_channel) \
spDeinterleaveData(idata, ilength, ddata, dlength, num_channel, float)
/** @~english Deinterleaves data of double type. */
#define spDeinterleaveDouble(idata, ilength, ddata, dlength, num_channel) \
spDeinterleaveData(idata, ilength, ddata, dlength, num_channel, double)
    
/** @~english @} */

/** @~english @name Text I/O
 *  @{ */
 
/** @~english Gets the number of rows of the text file.
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - A quoted string will be treated as a single column.
 - An escaped character by '\\' such as '\\#' will be converted into a non-escaped character such as '#'.
 */
extern int spGetNumRow(const char *filename);
/** @~english Gets the number of columns of the text file. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - A quoted string will be treated as a single column.
 - An escaped character by '\\' such as '\\#' will be converted into a non-escaped character such as '#'.
 */ 
extern int spGetNumColumn(const char *filename);

/** @~english Gets the number of rows of the CSV file.
 */
extern int spGetCSVNumRow(const char *filename);
/** @~english Gets the number of columns of the CSV file.
 */ 
extern int spGetCSVNumColumn(const char *filename);

/** @~english Gets a column from the text file. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - A quoted string will be treated as a single column.
 - An escaped character by '\\' such as '\\#' will be converted into a non-escaped character such as '#'.
 */
extern int spFGetNColumn(char *buf, int size, int col, FILE *fp);
/** @~english Gets a column from the text file. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - A quoted string will be treated as a single column.
 - An escaped character by '\\' such as '\\#' will be converted into a non-escaped character such as '#'.
 */
extern int spFGetColumn(char *buf, int col, FILE *fp);
/** @~english Gets a column from the specified line. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - A quoted string will be treated as a single column.
 - An escaped character by '\\' such as '\\#' will be converted into a non-escaped character such as '#'.
 */
extern int spSGetNColumn(char *buf, int size, int col, const char *line);
/** @~english Gets a column from the specified line. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - A quoted string will be treated as a single column.
 - An escaped character by '\\' such as '\\#' will be converted into a non-escaped character such as '#'.
 */
extern int spSGetColumn(char *buf, int col, const char *line);
/** @~english Gets a column from the specified line and returns the address of the next column. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - A quoted string will be treated as a single column.
 - An escaped character by '\\' such as '\\#' will be converted into a non-escaped character such as '#'.
 */
extern char *spSGetNextNColumn(char *buf, int size, const char *line);

/** @~english Gets a column from the CSV file.
 */
extern int spFGetCSVNColumn(char *buf, int size, int col, FILE *fp);
/** @~english Gets a column from the specified CSV line.
 */
extern int spSGetCSVNColumn(char *buf, int size, int col, const char *line);
/** @~english Gets a column from the specified CSV line and returns the address of the next column.
 */
extern char *spSGetNextCSVNColumn(char *buf, int size, const char *line);

/** @~english Gets a line from the text file. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - The line feed character such as '\\n' is removed.
 */
extern int spFGetNLine(char *buf, int size, FILE *fp);
/** @~english Gets a line from the text file. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - The line feed character such as '\\n' is removed.
 */
extern int spFGetLine(char *buf, FILE *fp); 
/** @~english Gets a line from the standard input. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - The line feed character such as '\\n' is removed.
 */
extern int spGetNLine(char *buf, int size);
/** @~english Gets a line from the standard input. This function reads data based on the following rules:
 - A comment from '#' to the line end is removed.
 - Multiple lines escaped by '\\' at the end of each line will be treated as a single line.
 - The line feed character such as '\\n' is removed.
 */
extern int spGetLine(char *buf);

/** @~english Puts a line to the text file. '#', '\\', '"' and "'" are escaped by '\\'.
 The line feed character is not implicitly added.
 */
extern int spFPutLine(const char *input, FILE *fp); /* the line feed character is not implicitly added */
/** @~english Puts a line to the buffer. '#', '\\', '"' and "'" are escaped by '\\'. 
 */
extern int spSPutLine(const char *input, char *line, int size);

/** @~english Puts a column to the text file. '#', '\\', '"' and "'" are escaped by '\\'.
 A string that includes white spaces or tabs will be quoted by '"'.
 */
extern int spFPutString(const char *input, FILE *fp);
/** @~english Puts a column to the buffer. '#', '\\', '"' and "'" are escaped by '\\'.
 A string that includes white spaces or tabs will be quoted by '"'.
 */
extern int spSPutString(const char *input, char *line, int size);

/** @~english Gets a string to the line end from the text file.
 Multiple lines escaped by '\' at the end of each line will be treated as a single line.
 */
extern char *spFGetNString(char *buf, int size, FILE *fp);
/** @~english Gets a string to the line end from the standard input.
 Multiple lines escaped by '\' at the end of each line will be treated as a single line.
 */
extern char *spGetNString(char *buf, int size);
/** @~english Gets a string to the line end from the buffer and returns the address of the next string.
 Multiple lines escaped by '\' at the end of each line will be treated as a single line.
 */
extern char *spSGetNString(char *buf, int size, const char *string);

/** @~english Extracts the parameter name and the parameter value from the line of the setup file. */
extern int spSScanNSetup(const char *line, char *name, int namelen, char *value, int valuelen);
/** @~english Extracts the parameter name and the parameter value from the line of the setup file. */
extern int spSScanSetup(const char *line, char *name, char *value);

/** @~english @} */

/** @internal */
extern unsigned char spLinear2ALaw(int pcm_val);
/** @internal */
extern int spALaw2Linear(unsigned char a_val);
/** @internal */
extern unsigned char spLinear2ULaw(int pcm_val);
/** @internal */
extern int spULaw2Linear(unsigned char u_val);
/** @internal */
extern unsigned char spALaw2ULaw(unsigned char aval);
/** @internal */
extern unsigned char spULaw2ALaw(unsigned char uval);

#if defined(MACOS)
#pragma import off
#endif

/** @} */  /*----@addtogroup spFileGroup----*/

#if !defined(SP_DISABLE_ABBREVIATION)
#define getfilesize spGetFileSize
#define getfilesize_txt spGetTextFileSize
#define fskipheader spFSkipHeader
#define fskipbyte spFSkipByte
#define shiftshort spShiftShort
#define shiftlong spShiftLong
#define shiftfloat spShiftFloat
#define shiftdouble spShiftDouble
#define swapshort spSwapShort
#define swaplong spSwapLong
#define swaplong32 spSwapLong32
#define swapulong32 spSwapULong32
#define swapfloat spSwapFloat
#define swapdouble spSwapDouble
#define freadshort spFReadShort
#define freadlong spFReadLong
#define freadlong32 spFReadLong32
#define freadulong32 spFReadULong32
#define freadlong24 spFReadLong24
#define freadulong24 spFReadULong24
#define freadlong64 spFReadLong64
#define freadulong64 spFReadULong64
#define freadfloatweighted spFReadFloatWeighted
#define freadfloat spFReadFloat
#define freaddoubleweighted spFReadDoubleWeighted
#define freaddouble spFReadDouble
#define freadbyte spFReadByte
#define freadsbyte spFReadSignedByte
#define freadulaw spFReadULaw
#define freadalaw spFReadALaw
#define freadshorttodweighted spFReadShortToDoubleWeighted
#define freadshorttod spFReadShortToDouble
#define freadlongtodweighted spFReadLongToDoubleWeighted
#define freadlongtod spFReadLongToDouble
#define freadlong32todweighted spFReadLong32ToDoubleWeighted
#define freadlong32tod spFReadLong32ToDouble
#define freadlong24todweighted spFReadLong24ToDoubleWeighted
#define freadlong24tod spFReadLong24ToDouble
#define freadfloattodweighted spFReadFloatToDoubleWeighted
#define freadfloattod spFReadFloatToDouble
#define freadbittodoubleweighted spFReadBitToDoubleWeighted
#define freadbittodouble spFReadBitToDouble
#define freadframebit spFReadFrameBit

#define fwriteshort spFWriteShort
#define fwritelong spFWriteLong
#define fwritelong32 spFWriteLong32
#define fwriteulong32 spFWriteULong32
#define fwritelong24 spFWriteLong24
#define fwriteulong24 spFWriteULong24
#define fwritelong64 spFWriteLong64
#define fwriteulong64 spFWriteULong64
#define fwritefloatweighted spFWriteFloatWeighted
#define fwritefloat spFWriteFloat
#define fwritedoubleweighted spFWriteDoubleWeighted
#define fwritedouble spFWriteDouble
#define fwritebyte spFWriteByte
#define fwritesbyte spFWriteSignedByte
#define fwriteulaw spFWriteULaw
#define fwritealaw spFWriteALaw
#define fwritedoubletosweighted spFWriteDoubleToShortWeighted
#define fwritedoubletos spFWriteDoubleToShort
#define fwritedoubletolweighted spFWriteDoubleToLongWeighted
#define fwritedoubletol spFWriteDoubleToLong
#define fwritedoubletol32weighted spFWriteDoubleToLong32Weighted
#define fwritedoubletol32 spFWriteDoubleToLong32
#define fwritedoubletol24weighted spFWriteDoubleToLong24Weighted
#define fwritedoubletol24 spFWriteDoubleToLong24
#define fwritedoubletofweighted spFWriteDoubleToFloatWeighted
#define fwritedoubletof spFWriteDoubleToFloat
#define fwritedoubletobitweighted spFWriteDoubleToBitWeighted
#define fwritedoubletobit spFWriteDoubleToBit
#define fwriteframebit spFWriteFrameBit
#define fwritelong32tos spFWriteLong32ToShort
#define fwritelong24tos spFWriteLong24ToShort

#define getnumrow spGetNumRow
#define getnumcol spGetNumColumn
#define fgetncol spFGetNColumn
#define fgetcol spFGetColumn
#define sgetncol spSGetNColumn
#define sgetcol spSGetColumn
#define sgetnextncol spSGetNextNColumn
#define fgetnline spFGetNLine
#define fgetline spFGetLine
#define getline spGetLine
#define fputline spFPutLine
#define sputline spSPutLine
#define fputstring spFPutString
#define sputstring spSPutString
#define fgetsn spFGetNString
#define getsn spGetNString
#define sgetsn spSGetNString
    
#define snscansetup spSScanNSetup
#define sscansetup spSScanSetup

#define linear2alaw spLinear2ALaw 
#define alaw2linear spALaw2Linear
#define linear2ulaw spLinear2ULaw
#define ulaw2linear spULaw2Linear
#define alaw2ulaw spALaw2ULaw
#define ulaw2alaw spULaw2ALaw

#define getsiglen spGetSignalLength
#define fskipdata spFSkipData

/* for backwards compatibility */
#define skipheader spFSkipHeader
#define skipbyte spFSkipByte
#define skipdata spFSkipData

#define getnumrow_txt spGetNumRow
#define getnumcol_txt spGetNumColumn

#define fgetcol_txt spFGetColumn
#define sgetcol_txt spSGetColumn

#define gets0 spGetNString
#endif /* !defined(SP_DISABLE_ABBREVIATION) */

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPFILE_H */
