/** @file sp/spPlugin.h
 */

#ifndef __SPPLUGIN_H
#define __SPPLUGIN_H

#include <sp/spDefs.h>
#include <sp/spOption.h>

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spPluginGroup Plugin */

/** @addtogroup spPluginGroup
 *  @{ */  /*----@addtogroup spPluginGroup----*/

/** @~english
 *  @class spPlugin spPlugin.h <sp/spPlugin.h>
 *  @brief A class to handle plugin-based processing.
@details Actually, spPlugin is not a class, but a typedefed type by `typedef struct _spPlugin spPlugin;`.
 */

#define SP_PLUGIN_VERSION_STRING	"1.7.1"
#define SP_PLUGIN_VERSION 		1
#define SP_PLUGIN_REVISION		7
#define SP_PLUGIN_UPDATE_LEVEL		1
#define SP_PLUGIN_VERSION_ID		(SP_PLUGIN_VERSION * 1000 + SP_PLUGIN_REVISION)

/** @~english Checks if the version of the spPlugin header files is the same as or newer than the specified version.
 *  @param[in] version The version number.
 *  @param[in] revision The revision number.
 *  @param[in] update The update number.
 */
#define SP_PLUGIN_CHECK_VERSION(version, revision, update) \
    (SP_PLUGIN_VERSION > (version) || \
     (SP_PLUGIN_VERSION == (version) && SP_PLUGIN_REVISION > (revision)) || \
     (SP_PLUGIN_VERSION == (version) && SP_PLUGIN_REVISION == (revision) && SP_PLUGIN_UPDATE_LEVEL >= (update)))
    
#define SP_PLUGIN_VERIFY_VERSION
#define SP_PLUGIN_SUPPORTED_VERSION_ID	/*SP_PLUGIN_VERSION_ID*/1003  /**< The version ID supported by these header files. */

/*
 * spPluginType: 	"plugin_type_name version_id"
 * version_id: 		version.revision
 */    
typedef const char *spPluginType;
#define SP_PLUGIN_UNKNOWN NULL
    
/** The callback reason for the plugin callback function. */
typedef enum {
    SP_PLUGIN_CR_NONE = -1,
    SP_PLUGIN_CR_ERROR = 0,     /**< Callbacked for an error.  */
    SP_PLUGIN_CR_OPTION = 1,    /**< Callbacked when the options must be updated. host_data: spOptions */
} spPluginCallbackReason;

/** The current state of the plugin. */
typedef enum {
    SP_PLUGIN_STATE_ERROR = -1, /**< Error. */
    SP_PLUGIN_STATE_STOP = 0,   /**< Stopped state. */
    SP_PLUGIN_STATE_START = 1,  /**< Started state. */
    SP_PLUGIN_STATE_PAUSE,      /**< Paused state. */
} spPluginState;

/** Target device of the plugin. */
typedef enum {
    SP_PLUGIN_DEVICE_UNKNOWN = -1, /**< Unknow target. */
    SP_PLUGIN_DEVICE_FILE = 0,     /**< A file. */
    SP_PLUGIN_DEVICE_AUDIO = 1,    /**< An audio device. */
    SP_PLUGIN_DEVICE_CD = 2,       /**< A CD player.  */
    SP_PLUGIN_DEVICE_MIXER = 3,    /**< A mixer device. */
    SP_PLUGIN_DEVICE_OTHERS = 4,   /**< Other devices. */
} spPluginDeviceType;

/* this parameter is used for finding file. */
#define SP_PLUGIN_PRIORITY_HIGHEST 0 /**< Highest plugin priority. */
#define SP_PLUGIN_PRIORITY_HIGHER 1  /**< Plugin priority higher than middle. */
#define SP_PLUGIN_PRIORITY_MIDDLE 2  /**< Middle plugin priority. */
#define SP_PLUGIN_PRIORITY_LOWER 3   /**< Plugin priority lower than middle. */
#define SP_PLUGIN_PRIORITY_LOWEST 4  /**< Lowest plugin priority. */
#define SP_PLUGIN_PRIORITY_NULL 5    /**< Plugin priority is not defined. */

/** Definition of spPluginError. */
typedef int spPluginError;
#define SP_PLUGIN_ERROR_SUCCESS SP_TRUE /**< Success. */
#define SP_PLUGIN_ERROR_FAILURE SP_FALSE /**< Failure. */
#define SP_PLUGIN_ERROR_NO_ERROR SP_TRUE /**< No error. */
#define SP_PLUGIN_ERROR_WRONG_PLUGIN (-1) /**< Wrong plugin. */
#define SP_PLUGIN_ERROR_OPEN (-2)         /**< Error in opening. */
#define SP_PLUGIN_ERROR_READ (-3)         /**< Error in reading. */
#define SP_PLUGIN_ERROR_WRITE (-4)        /**< Error in writing. */
#define SP_PLUGIN_ERROR_LOAD (-5)         /**< Error in loading. */
#define SP_PLUGIN_ERROR_SUITABLE_NOT_FOUND (-6) /**< No suitable plugin is found. */
#define SP_PLUGIN_ERROR_MULTIPLE_INSTANCE (-7)  /**< Doesn't allow multiple instanciations. */
#define SP_PLUGIN_ERROR_INSTANTIATE (-8)        /**< Error in instantiating. */
#define SP_PLUGIN_ERROR_BOGUS_FILE (-10)        /**< The file is broken. */
#define SP_PLUGIN_ERROR_FILE_TYPE (-11)         /**< Doesn't accept the file type. */
#define SP_PLUGIN_ERROR_SAMP_RATE (-12)         /**< Doesn't accept the sampling rate. */
#define SP_PLUGIN_ERROR_SAMP_BIT (-13)          /**< Doesn't accept the bits/sample.  */
#define SP_PLUGIN_ERROR_NUM_CHANNEL (-14)       /**< Doesn't accept the number of channels.  */
#define SP_PLUGIN_ERROR_OTHER_INFO (-15)        /**< Error caused by other information. */
#define SP_PLUGIN_ERROR_TOTAL_LENGTH_REQUIRED (-16) /**< The total length of the file must be specified. */
#define SP_PLUGIN_ERROR_UNKNOWN_DEVICE (-20)        /**< Doesn't recognize the device. */
#define SP_PLUGIN_ERROR_USER (-256)                 /**< User defined errors can be defined from this number. */

#define SP_PLUGIN_CAPS_THREAD_SAFE (1<<1) /**< The plugin is thread-safe. The thread-safe plugin must be run with multiple insntances. */
#define SP_PLUGIN_CAPS_NEED_LOCK (1<<2)   /**< The plugin needs locking. */
#define SP_PLUGIN_CAPS_MEMORY_SAFE (1<<3) /**< The plugin doesn't execute memory related functions in the read/write function. */
#define SP_PLUGIN_CAPS_KEEP_IN_MEMORY (1<<4) /**< The plugin doesn't need big memory. */

/* Version 1.4.1+ */    
#define SP_PLUGIN_CAPS_CUSTOM_OPTION_DIALOG (1<<5) /**< The plugin displays the option dialog specialized for the plugin. If this caps is turned on, A call of free_options with options=NULL must display the option dialog. */

/* Version 1.5.1+ */    
#define SP_PLUGIN_CAPS_THREAD_BY_THREAD_SAFE (1<<6) /**< The plugin is not true thread-safe, but thread-safe in a thread-by-thread basis. */

    
#define SP_PLUGIN_CAPS_SUBPLUGIN_SHIFT 12
#define SP_PLUGIN_CAPS_SUBPLUGIN (1<<SP_PLUGIN_CAPS_SUBPLUGIN_SHIFT)
    
#define SP_PLUGIN_CAPS_USER_SHIFT 16
#define SP_PLUGIN_CAPS_USER (1<<SP_PLUGIN_CAPS_USER_SHIFT)

#if defined(IPHONE)
#define spRegisterUniquePluginRec(unique_id)                         \
    {                                                                \
        extern spPluginRec *spGetPluginRec_ ## unique_id (long *, long *); \
        spPluginRec *rec;                                            \
        if ((rec = spCallUniqueGetPluginRecFunc( spGetPluginRec_ ## unique_id )) != NULL) { \
            spAddPluginRecList(rec, #unique_id );                    \
        }                                                            \
    }
#else
#define spRegisterUniquePluginRec(unique_id)
#endif

struct _spPluginRect
{
    short top;
    short left;
    short bottom;
    short right;
};
typedef struct _spPluginRect spPluginRect;
    
typedef struct _spPluginRec spPluginRec;
typedef struct _spPluginHost spPluginHost;
typedef struct _spPlugin spPlugin;

typedef struct _spIoPluginRec spIoPluginRec;
    
typedef spPluginRec *(*spGetPluginRecFunc)(void);
typedef long (*spPluginVersionFunc)(void);
typedef spPluginRec *(*spUniqueGetPluginRecFunc)(long *system_version_id, long *supported_version_id);

typedef spBool (*spPluginOpenCallback)(spPlugin *plugin, spPluginCallbackReason reason, void *host_data, void *call_data);

typedef spBool (*spSearchSpecificPluginCallback)(spPlugin *plugin, void *call_data);

#if defined(MACOS)
#pragma import on
#endif

/** @fn const char *spGetPluginSearchPath(void)
 *  @~english Gets the current search path of plugins.
 *  @return Returns the current null-terminated path of the directories. 
     Multiple directories are indicated by using the separator of `;' on Windows and Mac or `:' on others.

 ** @fn void spSetPluginSearchPath(const char *pathlist)
 *  @~english Sets the directories to search plugins.
 *  @param[in] pathlist The null-terminated path of the directories. This list overrides the current list.
     Multiple directories can be specified by using the separator of `;' on Windows and Mac or `:' on others.
     Specifiying NULL will initialize the current list.

 ** @fn spBool spAppendPluginSearchPath(const char *pathlist)
 *  @~english Appends the directories to the tail of the current search path of plugins.
 *  @param[in] pathlist The null-terminated path of the directories. This list will be appended to the tail of the current list.
     Multiple directories can be specified by using the separator of `;' on Windows and Mac or `:' on others.
     Specifiying NULL does not change the current list.

 ** @fn spBool spPrependPluginSearchPath(const char *pathlist)
 *  @~english Prepends the directories to the head of the current search path of plugins.
 *  @param[in] pathlist The null-terminated path of the directories. This list will be prepended to the head of the current list.
     Multiple directories can be specified by using the separator of `;' on Windows and Mac or `:' on others.
     Specifiying NULL does not change the current list.

 ** @fn spBool spIsPluginFile(const char *filename)
 *  @~english Checks whether the file is a plugin.
 This function checks that only by using the suffix of the file name, i.e., doesn't load the plugin.
 *  @param[in] filename The null-terminated path of the file.
 \p filename is not a body of the name but a actual path as opposed to spLoadPlugin() .
 *  @retval SP_TRUE Success  @retval SP_FALSE Failure

 ** @fn char *xspSearchPluginFile(int index)
 *  @~english Searches the \p index-th plugin and Gets the file name of the plugin.
   This function targets only the files that spIsPluginFile() returns SP_TRUE.
 *  @param[in] index The index for a plugin.
 *  @return A null-terminated string indicating the file name of the plugin.
 You have to deallocate the memory by calling xspFree() . NULL will be returned if error occurs.
 */

/** @} */  /*----@addtogroup spPluginGroup----*/

extern void spSetPluginLang(const char *lang);
extern const char *spGetPluginSearchPath(void);
extern void spSetPluginSearchPath(const char *pathlist); /* NULL pathlist clears current list */
extern spBool spAppendPluginSearchPath(const char *pathlist);
extern spBool spPrependPluginSearchPath(const char *pathlist);

extern spBool spAddPluginRecList(spPluginRec *plugin_rec, const char *name);
extern spBool spRemovePluginRecList(spPluginRec *plugin_rec);
extern spBool _spRegisterPluginRec(spPluginRec *plugin_rec, const char *name, long system_version_id, long supported_version_id);
#define spRegisterPluginRec(plugin_rec, name) _spRegisterPluginRec(plugin_rec, name, SP_PLUGIN_SUPPORTED_VERSION_ID, SP_PLUGIN_VERSION_ID)
extern spPlugin *spAllocNullPlugin(void);
extern spPluginRec *spCallUniqueGetPluginRecFunc(spUniqueGetPluginRecFunc func);

/** @~english Loads a plugin. If you call this function before calling spFreePlugin() ,
 the identical address of the plugin that has already loaded.
 In this case, the number of calls of spFreePlugin() must be balanced with that of spLoadPlugin() .
 *  @param[in] plugin_name The null-terminated name of the plugin excluding the suffix.
 The directories specified by spSetPluginSearchPath() or the default directories are searched.
 *  @return Returns the pointer to spPlugin type. You have to unload the plugin by calling spFreePlugin() .
 NULL will be returned if error occurs.
 *  @public @memberof spPlugin */
extern spPlugin *spLoadPlugin(const char *plugin_name);
/** @~english Unloads the plugin. If you call spLoadPlugin() multiple times,
 the number of calls of spFreePlugin() must be balanced with that of spLoadPlugin() .
 *  @param[in] plugin The pointer to spPlugin type obtained by spLoadPlugin() .
 *  @retval SP_TRUE Success  @retval SP_FALSE Failure
 *  @public @memberof spPlugin */
extern spBool spFreePlugin(spPlugin *plugin);

/** @~english Checks the type of the plugin.
 *  @public @memberof spPlugin */
extern spBool spEqPluginType(spPlugin *plugin, spPluginType type);
/** @~english Gets the name of the plugin.
 *  @public @memberof spPlugin */
extern const char *spGetPluginName(spPlugin *plugin);
/** @~english Gets the ID of the plugin.
 *  @public @memberof spPlugin */
extern const char *spGetPluginId(spPlugin *plugin);
/** @~english Gets the short description of the plugin which doesn't include line breaks.
 *  @public @memberof spPlugin */
extern const char *spGetPluginDescription(spPlugin *plugin);
/** @~english Gets the detailed description of the plugin which can include line breaks.
 *  @public @memberof spPlugin */
extern const char *spGetPluginInformation(spPlugin *plugin);
/** @~english Gets the version ID of the plugin which is an integer of long type.
 *  @public @memberof spPlugin */
extern long spGetPluginVersionId(spPlugin *plugin);
/** @~english Gets the priority of the plugin to determine which plugin is used for a purpose,
 when several plugins for the purpose are found.
 #SP_PLUGIN_PRIORITY_HIGHEST is the highest and #SP_PLUGIN_PRIORITY_LOWEST is the lowest priority.
 *  @public @memberof spPlugin */
extern int spGetPluginPriority(spPlugin *plugin);
/** @~english Gets the version number of the plugin loaded.
 *  @public @memberof spPlugin */
extern spBool spGetPluginVersion(spPlugin *plugin, int *version, int *revision);
extern unsigned long spGetPluginCaps(spPlugin *plugin);
extern spBool spIsPluginCapable(spPlugin *plugin, unsigned long cap);

extern void *spGetPluginInstance(spPlugin *plugin);
extern long spGetNumPluginInstance(spPlugin *plugin);
extern spBool spIsPluginInstantiatable(spPlugin *plugin);
extern spBool spInitPluginInstance(spPlugin *plugin);
extern spBool spFreePluginInstance(spPlugin *plugin);
extern spPlugin *spInstantiatePlugin(const char *plugin_name);

extern spOptions spInitPluginOptions(spPlugin *plugin);
extern spBool spFreePluginOptions(spPlugin *plugin, spOptions options);
extern spBool spDisplayPluginCustomOptionDialog(spPlugin *plugin);
extern void spPrintPluginOptions(spPlugin *plugin);
extern spBool spPluginUsage(const char *plugin_name);

extern spBool spIsPluginFile(const char *filename);
extern const char *spSearchPluginFile(int index);
extern char *xspSearchPluginFile(int index);
extern void spFreePluginGlobals(void);

extern spBool spEqPluginNameString(const char *name1, const char *name2);
extern spBool spEqPluginName(spPlugin *plugin, const char *name);
extern spPlugin *spSearchSpecificPlugin(int *index, spSearchSpecificPluginCallback call_func, void *call_data);
extern char *xspSearchSpecificPluginFile(int *index, spSearchSpecificPluginCallback call_func, void *call_data);
extern int spGetNumSpecificPlugin(spSearchSpecificPluginCallback call_func, void *call_data);

#if defined(MACOS)
#pragma import off
#endif

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPPLUGIN_H */
