/** @file sp/spThread.h
 */

#ifndef __SPTHREAD_H
#define __SPTHREAD_H

#include <sp/spDefs.h>

#if defined(_WIN32) && !defined(HAVE_PTHREAD)
#if defined(USE_MOTIF) || defined(GTK)
#undef Status
#define INT32 wINT32
#define boolean wboolean
#endif
#include <windows.h>
#if defined(_MSC_VER)
#include <process.h>
#define HAVE_BEGINTHREADEX
#endif /* defined(_MSC_VER) */
#if defined(USE_MOTIF) || defined(USE_GTK)
#define Status int
#undef INT32
#undef boolean
#endif
#elif defined(MACOS) && !defined(MACOSX)
#include <Threads.h>
#elif defined(MACOSX) || defined(HAVE_PTHREAD)
#include <pthread.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/** @~english @defgroup spThreadGroup Multithreading */

/** @addtogroup spThreadGroup
 *  @{ */  /*----@addtogroup spThreadGroup----*/

#define SP_THREAD_PRIORITY_TIME_CRITICAL 15     /**< Thread priority for really time critical processing (higher than the highest thread priority).  */
#define SP_THREAD_PRIORITY_HIGHEST 2            /**< Highest thread priority. */
#define SP_THREAD_PRIORITY_ABOVE_NORMAL 1       /**< Thread priority above normal. */
#define SP_THREAD_PRIORITY_NORMAL 0             /**< Normal thread priority. */
#define SP_THREAD_PRIORITY_BELOW_NORMAL (-1)    /**< Thread priority below normal. */
#define SP_THREAD_PRIORITY_LOWEST (-2)          /**< Lowest thread priority. */
#define SP_THREAD_PRIORITY_IDLE (-15)           /**< Thread priority for idle state (lower than the lowest thread priority).  */

#define SP_THREAD_EXIT_SUCCESS 0
#define SP_THREAD_EXIT_FAILURE -2
#define SP_THREAD_EXIT_CANCELED -1

#define SP_EVENT_TIMEOUT_INFINITE -1    /**< Infinite timeout for spWaitEventTimeout() . */
#define SP_EVENT_WAIT_FAILED -1         /**< Waiting for the event object failed. */
#define SP_EVENT_WAIT_SIGNALED 0        /**< The event object signaled. */
#define SP_EVENT_WAIT_TIMEOUT 1         /**< Timeout period elapsed. */
#define SP_EVENT_WAIT_ABANDONED 2       /**< Waiting for the event object is abandoned. */

typedef enum {
    SP_THREAD_LEVEL_LOW = 0,
    SP_THREAD_LEVEL_BELOW_MIDDLE = 25,
    SP_THREAD_LEVEL_MIDDLE = 50,
    SP_THREAD_LEVEL_ABOVE_MIDDLE = 75,
    SP_THREAD_LEVEL_HIGH = 100,
} spThreadLevel;

/** @} */  /*----@addtogroup spThreadGroup----*/

#if defined(_WIN32) && !defined(HAVE_PTHREAD)
#if defined(HAVE_BEGINTHREADEX)
/** @internal */
#define spThreadReturn unsigned __stdcall
/** @internal */
typedef unsigned (__stdcall *spThreadFunc)(void *data);
#else
/** @internal */
#define spThreadReturn DWORD WINAPI
/** @internal */
typedef DWORD (WINAPI *spThreadFunc)(void *data);
#endif

#define SP_THREAD_RETURN_SUCCESS 0
#define SP_THREAD_RETURN_FAILURE 1

/** @internal */
#define spThreadId DWORD
    
#else
/** @internal Definition of spThreadReturn for pthread and MacOS. */
#define spThreadReturn void *
typedef spThreadReturn (*spThreadFunc)(void *data);

#define SP_THREAD_RETURN_SUCCESS (void *)0
#define SP_THREAD_RETURN_FAILURE (void *)1

#if defined(MACOS) && !defined(MACOSX)
/** @internal */
#define spThreadId ThreadID
#else
/** @internal */
#define spThreadId pthread_t
#endif
    
#endif

/** @addtogroup spThreadGroup
 *  @{ */

/** Status of the command thread. This status can be accessed as the 1st argument of #spCommandThreadCallback . */
typedef enum {
    SP_COMMAND_THREAD_STARTED = 0, 		/**< Status when the command thread started. data: 0 */
    SP_COMMAND_THREAD_FINISHED = 1,		/**< Status when the command thread finished. data: 0 */
    SP_COMMAND_THREAD_COMMAND_FINISHED = 2, 	/**< Status when the specified command finished. data: command exit code */
    SP_COMMAND_THREAD_COMMAND_FAILED = 3, 	/**< Status when the specified command failed. data: errno */
    SP_COMMAND_THREAD_COMMAND_WAIT_FAILED = 4, 	/**< Status when waiting for the specified command failed. data: errno */
} spCommandThreadStatus;

/** Signature of the callback function for spCreateCommandThread() . */
typedef void (*spCommandThreadCallback)(spCommandThreadStatus status, long data, void *user_data);

    
#if defined(MACOS)
#pragma import on
extern int spGetNumCoopThreadMac(void);
#endif

/** @~english Creates a new thread. The created thread must be destroyed by spDestroyThread() . */
extern void *spCreateThread(long stacksize, int priority, spThreadFunc func, void *data);
/** @~english Destroys the thread created by spCreateThread() . */
extern void spDestroyThread(void *handle);
/** @~english Exits the current thread created by spCreateThread() .
 *  The behavior of calling this function from the thread created by a function other than spCreatedThread() is undefined. */
extern void spExitThread(long status);
/** @~english Yields ownership of the current thread to other threads. This function is valid for environment that can gain only one ownership simultaneously such as Mac OS 9. */
extern void spYieldThread(void);
/** @~english Waits the thread created by spCreateThread() . */
extern long spWaitThread(void *handle);
    
/** @~english Gets the ID of the thread. */
extern spThreadId spGetThreadId(void *handle);
/** @~english Gets the ID of the current thread. */
extern spThreadId spGetCurrentThreadId(void);
/** @~english Check whether two thread IDs are the same. */
extern spBool spEqThreadId(spThreadId id1, spThreadId id2);

/** @~english Creates a new mutex object. */
extern void *spCreateMutex(const char *name);
/** @~english Destroyes the mutex object created by spCreateMutex() . */
extern void spDestroyMutex(void *handle);
/** @~english Gains ownership of the mutex object. */
extern spBool spLockMutex(void *handle);
/** @~english Relinquishes ownership of the mutex object. */
extern spBool spUnlockMutex(void *handle);
/** @~english Waits for access to the mutex object. */
extern spBool spWaitMutex(void *handle);

/** @~english Creates a new event object. */
extern void *spCreateEvent(spBool manual_reset, spBool initial_signal);
/** @~english Destroyes the event object created by spCreateEvent() . */
extern spBool spDestroyEvent(void *handle);
/** @~english Waits for the event object to be a signaled state. */
extern int spWaitEventTimeout(void *handle, long timeout_ms);
/** @~english Waits for the event object to be a signaled state. */
extern spBool spWaitEvent(void *handle);
/** @~english Sets the event object to a signaled state. */
extern spBool spSetEvent(void *handle);
/** @~english Resets the event object to a nonsignaled state. */
extern spBool spResetEvent(void *handle);
/** @~english Sets the event object to a signaled state, and then promptly resets that to a nonsignaled state. */
extern spBool spPulseEvent(void *handle);
    
extern spBool spIsThreadLevelVariable(void);
extern spBool spIsLowLevelThreadUsed(void);
extern spBool spSetThreadLevel(spThreadLevel level);
extern spThreadLevel spGetThreadLevel(void);
    
/** @~english Executes the command in a thread.
 *  @param[in] command The null-terminated command executed. 
 *  @param[in] func The callback function which has the following signature ( #spCommandThreadCallback ): @code
      void func(spCommandThreadStatus status, long data, void *user_data);
 *  @endcode
 *  @param[in] data The user data for the callback function. You can obtain this address as the 3rd argument of the callback function.
 *  @retval SP_TRUE Success  @retval SP_FALSE Failure  */
extern spBool spCreateCommandThread(const char *command, spCommandThreadCallback func, void *data);

#if defined(MACOS)
#pragma import off
#endif

/** @} */  /*----@addtogroup spThreadGroup----*/

#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __SPTHREAD_H */
